"""
Permission model for the TNNT Discord bot.
"""

# Django
from django.db import models
from django.utils.translation import gettext_lazy as _


class Permission(models.Model):
    """
    Meta model for app permissions
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta class for the Permission model.
        """

        managed = False
        default_permissions = ()
        permissions = (
            ("locate", _("Can run the `/locate` command")),
            ("lookup", _("Can run the `/lookup` command")),
        )
        verbose_name = _("Command Permission")



# Generated by Django 4.2.23 on 2025-06-20 09:52

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("aadiscordbot", "0016_authbotconfiguration_admin_user_ids_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Setting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "applicant_role_name",
                    models.CharField(
                        default="",
                        help_text="The name of the role for applicants in the Discord server.",
                        max_length=100,
                        verbose_name="Applicant Role Name",
                    ),
                ),
                (
                    "applicant_role_id",
                    models.PositiveBigIntegerField(
                        default=None,
                        help_text="The ID of the role for applicants in the Discord server.",
                        null=True,
                        verbose_name="Applicant Role ID",
                    ),
                ),
                (
                    "recruiter_role_id",
                    models.PositiveBigIntegerField(
                        default=None,
                        help_text="The ID of the role for recruiters in the Discord server.",
                        null=True,
                        verbose_name="Recruiter Role ID",
                    ),
                ),
                (
                    "leadership_role_id",
                    models.PositiveBigIntegerField(
                        default=None,
                        help_text="The ID of the role for leadership in the Discord server.",
                        null=True,
                        verbose_name="Leadership Role ID",
                    ),
                ),
                (
                    "welcome_roles_excluded",
                    models.CharField(
                        default="Member",
                        help_text="Comma-separated list of Discord roles that are excluded from the welcome message.",
                        max_length=255,
                        verbose_name="Roles Excluded",
                    ),
                ),
                (
                    "recruiting_channel",
                    models.ForeignKey(
                        help_text="The channel used for recruiting in the Discord server.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="aadiscordbot.channels",
                        verbose_name="Recruiting Channel",
                    ),
                ),
                (
                    "welcome_channel_authenticated",
                    models.ForeignKey(
                        help_text="The channel to send welcome messages for authenticated users in the Discord server.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="aadiscordbot.channels",
                        verbose_name="Welcome Channel (Authenticated User)",
                    ),
                ),
                (
                    "welcome_channel_unauthenticated",
                    models.ForeignKey(
                        help_text="The channel to send welcome messages for unauthenticated users in the Discord server.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="aadiscordbot.channels",
                        verbose_name="Welcome Channel (Unauthenticated User)",
                    ),
                ),
            ],
            options={
                "verbose_name": "Setting",
                "verbose_name_plural": "Settings",
                "default_permissions": (),
            },
        ),
    ]



"""
RecruitMe Cog
"""

# Standard Library
import logging
from enum import Enum
from typing import Any

# Third Party
from discord import ButtonStyle, ChannelType, Embed, Guild, Interaction, Member, ui
from discord.ext import commands

# Django
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth Discord Bot
from aadiscordbot import app_settings
from aadiscordbot.utils import auth

# Terra Nanotech Discordbot Cogs
from tnnt_discordbot_cogs.helper import reverse_absolute, unload_cog
from tnnt_discordbot_cogs.models.setting import Setting

logger = logging.getLogger(__name__)

# Discord Bot Settings
# Audit System
AUDIT_SYSTEM_NAME = "Character Audit"
AUDIT_SYSTEM_URL = reverse_absolute(viewname="corptools:react")
AUDIT_SYSTEM_URL_MD = f"[{AUDIT_SYSTEM_NAME}]({AUDIT_SYSTEM_URL})"

# Groups Page URL
GROUPS_PAGE_URL = reverse_absolute(viewname="groupmanagement:groups")

# Roles
APPLICANT_ROLE_NAME = Setting.get_setting(Setting.Field.APPLICANT_ROLE_NAME.value)
APPLICANT_ROLE_ID = Setting.get_setting(Setting.Field.APPLICANT_ROLE_ID.value)
RECRUITER_ROLE_ID = Setting.get_setting(Setting.Field.RECRUITER_ROLE_ID.value)
LEADERSHIP_ROLE_ID = Setting.get_setting(Setting.Field.LEADERSHIP_ROLE_ID.value)

# Channels
RECRUITING_CHANNEL = Setting.get_setting(Setting.Field.RECRUITING_CHANNEL.value).channel


class BotResponse(str, Enum):
    """
    Bot responses for the RecruitMe cog
    """

    # Not in the application group
    NOT_IN_APPLICATION_GROUP = (
        f"**You are not in the `{APPLICANT_ROLE_NAME}` group.**\n\n"
        f"Please open the [groups page]({GROUPS_PAGE_URL}) and "
        f"join the `{APPLICANT_ROLE_NAME}` group first …"
    )

    # Member not in the application group
    MEMBER_NOT_IN_APPLICATION_GROUP = (
        "**{MEMBER} is not in the "
        f"`{APPLICANT_ROLE_NAME}` group.**\n\n"
        "Please let them know to open the "
        f"[groups page]({GROUPS_PAGE_URL}) and join the "
        f"`{APPLICANT_ROLE_NAME}` group first …"
    )

    # Not a recruiter
    NOT_A_RECRUITER = (
        "You are not a recruiter for Terra Nanotech and "
        "cannot use this command on this user …"
    )

    # Private thread guide title and body
    PRIVATE_THREAD_GUIDE_TITLE = "Private Thread Guide"
    PRIVATE_THREAD_GUIDE_BODY = (
        "To add a person to this thread simply `@ping` them. "
        "This works with `@groups` as well to bulk add people to the channel. "
        "Use wisely, abuse will not be tolerated."
    )

    # Recruitment thread title and body
    RECRUITMENT_THREAD_TITLE = "{MAIN_CHARACTER} | Recruitment | {DATE}"
    RECRUITMENT_THREAD_WELCOME = (
        "Hello <@{MEMBER_ID}>, and welcome! :wave:\n\n"
        "We're excited that you're interested in joining Terra Nanotech!\n\n"
        f"Before we get started, please ensure **all** your characters are added to {AUDIT_SYSTEM_URL_MD}.\n"
        "This includes your main character as well as any other characters you have.\n\n"
        'Once done, please click the "**Continue**" button below to notify a recruiter.\n'
        "If you are not certain yet and just want to ask some questions first to help "
        'you to decide, click the "**Omit This Step**" button.'
    )
    RECRUITMENT_THREAD_COMPLIANCE_REMINDER = (
        "Before we can proceed with your recruitment, "
        f"please ensure **all** your characters are added to {AUDIT_SYSTEM_URL_MD}.\n"
        "This includes your main character as well as any other characters you have.\n\n"
        # "The following characters are not compliant:\n"
        # "{CHARACTERS_LIST}\n\n"
        'Once done, please click the "**Continue**" button below to notify a recruiter.\n\n'
        "Thank you for your understanding!"
    )
    RECRUITMENT_THREAD_RECRUITING_START = (
        "Thank you for reaching out!\n\n"
        f"A member of the <@&{RECRUITER_ROLE_ID}> will be with you shortly to "
        "guide you through the next steps of the recruitment process, which "
        "may include an interview for further assessments.\n\n"
        "You may start the conversation by introducing yourself and tell us a little bit about you.\n"
        "- Who are you?\n"
        "- What are your experiences in EVE Online so far? (Tell us a bit about your journey/history in EVE)\n"
        "- Why would we consider you?\n"
        "- What makes you interesting?\n"
        "- What would you bring to the table?\n"
        "- In which way the corp would benefit from you?\n"
        "- What do you expect?\n\n"
        "And of cource, if you have any questions, feel free to ask!\n\n"
        "Thank you for your patience!\n\n"
        f"(Adding <@&{LEADERSHIP_ROLE_ID}> to the thread for visibility.)"
    )
    RECRUITMENT_THREAD_QUESTIONS_ONLY_START = (
        "Thank you for reaching out!\n\n"
        f"A member of the <@&{RECRUITER_ROLE_ID}> will be with you shortly "
        "to answer any questions you may have.\n\n"
        "Thank you for your patience!\n\n"
        f"(Adding <@&{LEADERSHIP_ROLE_ID}> to the thread for visibility.)"
    )
    RECRUITMENT_THREAD_CLOSURE = (
        "Your recruitment thread has been closed.\n\n"
        "If you change your mind, feel free to use the `/recruit_me` command again!"
    )

    # Recruitment thread created message
    RECRUITMENT_THREAD_CREATED = "Recruitment thread created!"


def _get_auth_user_and_main_character(
    discord_user_id: int, guild: Guild
) -> tuple[User, EveCharacter]:
    """
    Get the auth user and main character for a member

    :param discord_user_id:
    :type discord_user_id:
    :param guild:
    :type guild:
    :return:
    :rtype:
    """

    auth_user = auth.get_auth_user(user=discord_user_id, guild=guild)
    main_character = auth_user.profile.main_character

    return auth_user, main_character


def _check_compliance(auth_user: User) -> tuple[bool, Any]:
    """
    Check if all characters of the auth user are in the audit system

    :param auth_user:
    :type auth_user:
    :return:
    :rtype:
    """

    character_ownerships = auth_user.character_ownerships.all().select_related(
        "character", "character__characteraudit"
    )
    characters = []

    for character in character_ownerships:
        # if character.user.id not in characters:
        #     characters[character.user.id] = []

        try:
            # if not character.character.characteraudit.is_active():
            if not character.character.characteraudit:
                # characters[character.user.id].append(character.character.character_name)
                characters.append(character.character.character_name)
        except ObjectDoesNotExist:
            characters.append(character.character.character_name)

    return True if len(characters) == 0 else False, characters


class ComplianceView(ui.View):
    """
    View for compliance check
    """

    def __init__(self):
        """
        Initialize the ComplianceView
        """

        super().__init__(timeout=None)  # No timeout

        self.embed = Embed(
            title=BotResponse.PRIVATE_THREAD_GUIDE_TITLE.value,
            description=BotResponse.PRIVATE_THREAD_GUIDE_BODY.value,
        )

    @ui.button(
        label="Continue",
        row=0,
        style=ButtonStyle.success,
        custom_id="tnnt_cogs:recruitment:compliance_view:button:continue",
    )
    async def continue_button_callback(self, button, interaction):
        """
        Continue button callback

        :param button:
        :type button:
        :param interaction:
        :type interaction:
        :return:
        :rtype:
        """

        self.disable_all_items()

        await interaction.response.edit_message(view=self)

        auth_user, main_character = _get_auth_user_and_main_character(
            discord_user_id=interaction.user.id, guild=interaction.guild
        )

        is_compliant, non_compliant_characters = _check_compliance(auth_user)

        if not is_compliant:
            characters_list = "\n".join(
                f"- {char}" for char in non_compliant_characters
            )

            await interaction.channel.send(
                content=BotResponse.RECRUITMENT_THREAD_COMPLIANCE_REMINDER.value.format(
                    CHARACTERS_LIST=characters_list
                ),
                view=ComplianceView(),
            )
        else:
            await interaction.channel.send(
                content=BotResponse.RECRUITMENT_THREAD_RECRUITING_START.value.format(
                    LEADERSHIP_ROLE_ID=LEADERSHIP_ROLE_ID,
                    RECRUITER_ROLE_ID=RECRUITER_ROLE_ID,
                ),
                embed=self.embed,
            )

    @ui.button(
        label="Close Thread",
        row=0,
        style=ButtonStyle.danger,
        custom_id="tnnt_cogs:recruitment:compliance_view:button:close_thread",
    )
    async def close_thread_button_callback(self, button, interaction):
        """
        Close thread button callback

        :param button:
        :type button:
        :param interaction:
        :type interaction:
        :return:
        :rtype:
        """

        self.disable_all_items()

        await interaction.response.edit_message(view=self)
        await interaction.channel.send(
            content=BotResponse.RECRUITMENT_THREAD_CLOSURE.value
        )

        return await interaction.channel.archive()

    # @ui.button(
    #     label="Identify Me!",
    #     row=0,
    #     style=ButtonStyle.danger,
    #     custom_id="tnnt_cogs:recruitment:compliance_view:button:identify_me",
    # )
    # async def identify_me_button_callback(self, button, interaction):
    #     """
    #     Identify me button callback
    #
    #     :param button:
    #     :type button:
    #     :param interaction:
    #     :type interaction:
    #     :return:
    #     :rtype:
    #     """
    #
    #     await interaction.response.edit_message(view=self)
    #     await interaction.channel.send(
    #         content=f"Hello <@{interaction.user.id}>! You have identified yourself in this recruitment thread."
    #     )
    #     await interaction.channel.send(
    #         content=('Channel Title: "' + interaction.channel.name + '"')
    #     )
    #
    #     auth_user, main_character = _get_auth_user_and_main_character(
    #         discord_user_id=interaction.user.id, guild=interaction.guild
    #     )
    #
    #     await interaction.channel.send(
    #         content=('Your main character is: "' + main_character.character_name + '"')
    #     )


class RecruitmentThreadIntroduction(ui.View):
    """
    View for the recruitment thread introduction
    """

    # def __init__(self, auth_user: User):
    def __init__(self):
        """
        Initialize the RecruitmentThreadIntroduction view
        """

        super().__init__(timeout=None)

        self.embed = Embed(
            title=BotResponse.PRIVATE_THREAD_GUIDE_TITLE.value,
            description=BotResponse.PRIVATE_THREAD_GUIDE_BODY.value,
        )

    @ui.button(
        label="Continue",
        row=0,
        style=ButtonStyle.success,
        custom_id="tnnt_cogs:recruitment:recruitment_threat_introduction_view:button:continue",
    )
    async def continue_button_callback(self, button, interaction):
        """
        Continue button callback

        :param button:
        :type button:
        :param interaction:
        :type interaction:
        :return:
        :rtype:
        """

        self.disable_all_items()

        await interaction.response.edit_message(view=self)

        auth_user, main_character = _get_auth_user_and_main_character(
            discord_user_id=interaction.user.id, guild=interaction.guild
        )

        is_compliant, non_compliant_characters = _check_compliance(auth_user)

        if not is_compliant:
            characters_list = "\n".join(
                f"- {char}" for char in non_compliant_characters
            )

            await interaction.channel.send(
                content=BotResponse.RECRUITMENT_THREAD_COMPLIANCE_REMINDER.value.format(
                    CHARACTERS_LIST=characters_list
                ),
                view=ComplianceView(),
            )
        else:
            await interaction.channel.send(
                content=BotResponse.RECRUITMENT_THREAD_RECRUITING_START.value.format(
                    LEADERSHIP_ROLE_ID=LEADERSHIP_ROLE_ID,
                    RECRUITER_ROLE_ID=RECRUITER_ROLE_ID,
                ),
                embed=self.embed,
            )

    @ui.button(
        label="Omit This Step",
        row=0,
        style=ButtonStyle.secondary,
        custom_id="tnnt_cogs:recruitment:recruitment_threat_introduction_view:button:omit_this_step",
    )
    async def omit_button_callback(self, button, interaction):
        """
        Omit this step button callback

        :param button:
        :type button:
        :param interaction:
        :type interaction:
        :return:
        :rtype:
        """

        self.disable_all_items()

        await interaction.response.edit_message(view=self)
        await interaction.channel.send(
            content=BotResponse.RECRUITMENT_THREAD_QUESTIONS_ONLY_START.value.format(
                LEADERSHIP_ROLE_ID=LEADERSHIP_ROLE_ID,
                RECRUITER_ROLE_ID=RECRUITER_ROLE_ID,
            ),
            embed=self.embed,
        )

    @ui.button(
        label="Close Thread",
        row=0,
        style=ButtonStyle.danger,
        custom_id="tnnt_cogs:recruitment:recruitment_threat_introduction_view:button:close_thread",
    )
    async def close_thread_button_callback(self, button, interaction):
        """
        Close thread button callback

        :param button:
        :type button:
        :param interaction:
        :type interaction:
        :return:
        :rtype:
        """

        self.disable_all_items()

        await interaction.response.edit_message(view=self)
        await interaction.channel.send(
            content=BotResponse.RECRUITMENT_THREAD_CLOSURE.value
        )

        return await interaction.channel.archive()

    # @ui.button(
    #     label="Identify Me!",
    #     row=0,
    #     style=ButtonStyle.danger,
    #     custom_id="tnnt_cogs:recruitment:recruitment_threat_introduction_view:button:identify_me",
    # )
    # async def identify_me_button_callback(self, button, interaction):
    #     """
    #     Identify me button callback
    #
    #     :param button:
    #     :type button:
    #     :param interaction:
    #     :type interaction:
    #     :return:
    #     :rtype:
    #     """
    #
    #     await interaction.response.edit_message(view=self)
    #     await interaction.channel.send(
    #         content=f"Hello <@{interaction.user.id}>! You have identified yourself in this recruitment thread."
    #     )
    #     await interaction.channel.send(
    #         content=('Channel Title: "' + interaction.channel.name + '"')
    #     )
    #
    #     auth_user, main_character = _get_auth_user_and_main_character(
    #         discord_user_id=interaction.user.id, guild=interaction.guild
    #     )
    #
    #     await interaction.channel.send(
    #         content=('Your main character is: "' + main_character.character_name + '"')
    #     )


class RecruitMe(commands.Cog):
    """
    Thread Tools for recruiting!
    """

    def __init__(self, bot):
        """
        Initialize the RecruitMe cog

        :param bot: The bot instance
        :type bot: discord.ext.commands.Bot
        """

        logger.info(msg="RecruitMe cog initialized")

        self.bot = bot
        self._on_ready_done = False
        self.recruiting_channel = (
            None  # Optional: Resolved channel reference available after ready
        )

    @commands.Cog.listener()
    async def on_ready(self):
        # `on_ready` can fire multiple times; run initialization once
        if self._on_ready_done:
            return

        # Ensure bot is ready (usually redundant inside on_ready)
        if not self.bot.is_ready():
            await self.bot.wait_until_ready()

        logger.info("RecruitMe cog received on_ready")

        # Resolve the recruiting channel id into a Channel object for later use
        try:
            self.recruiting_channel = self.bot.get_channel(RECRUITING_CHANNEL)

            logger.info("Recruiting channel resolved: %s", self.recruiting_channel)
        except Exception:
            self.recruiting_channel = None

            logger.exception("Failed to resolve recruiting channel")

        # Register RecruitmentThreadIntroduction view
        try:
            self.bot.add_view(RecruitmentThreadIntroduction())

            logger.info("RecruitmentThreadIntroduction view added successfully")
        except Exception:
            logger.exception("Failed to add RecruitmentThreadIntroduction view")

        # Register ComplianceView view
        try:
            self.bot.add_view(ComplianceView())

            logger.info("ComplianceView view added successfully")
        except Exception:
            logger.exception("Failed to add ComplianceView view")

        # Mark initialization done so this runs only once
        self._on_ready_done = True

        logger.info("RecruitMe cog on_ready processing complete")

    @staticmethod
    async def open_ticket(ctx: Interaction, member: Member):
        """
        Open a recruitment ticket

        :param ctx: The context of the command
        :type ctx: discord.Interaction
        :param member: The member requesting a recruiter
        :type member: discord.Member
        :return: None
        :rtype: None
        """

        # Get the auth user and main character
        auth_user, main_character = _get_auth_user_and_main_character(
            discord_user_id=member.id, guild=ctx.guild
        )

        # Get the recruiting channel
        ch = ctx.guild.get_channel(RECRUITING_CHANNEL)

        # Create the recruitment thread
        th = await ch.create_thread(
            name=BotResponse.RECRUITMENT_THREAD_TITLE.value.format(
                MAIN_CHARACTER=main_character,
                DATE=timezone.now().strftime("%Y-%m-%d %H:%M"),
            ),
            auto_archive_duration=10080,
            type=ChannelType.private_thread,
            reason=None,
        )

        # Prepare the welcome message
        msg = BotResponse.RECRUITMENT_THREAD_WELCOME.value.format(
            LEADERSHIP_ROLE_ID=LEADERSHIP_ROLE_ID,
            RECRUITER_ROLE_ID=RECRUITER_ROLE_ID,
            MEMBER_ID=member.id,
        )

        # Send the welcome message in the recruitment thread
        await th.send(content=msg, view=RecruitmentThreadIntroduction())

        # Notify the user that their recruitment thread has been created
        await ctx.response.send_message(
            content=BotResponse.RECRUITMENT_THREAD_CREATED.value,
            view=None,
            ephemeral=True,
        )

    @commands.slash_command(
        name="recruit_me",
        description="Get hold of a recruiter",
        guild_ids=app_settings.get_all_servers(),
    )
    async def slash_recruit_me(self, ctx):
        """
        Get hold of a recruiter

        :param ctx: The context of the command
        :type ctx: discord.Interaction
        :return: None
        :rtype: None
        """

        if APPLICANT_ROLE_ID not in [role.id for role in ctx.user.roles]:
            return await ctx.respond(
                BotResponse.NOT_IN_APPLICATION_GROUP.value, ephemeral=True
            )

        await self.open_ticket(ctx=ctx, member=ctx.user)

        return None

    @commands.message_command(
        name="Create Recruitment Thread", guild_ids=app_settings.get_all_servers()
    )
    async def reverse_recruit_msg_context(self, ctx, message):
        """
        Help a new guy get a recruiter

        :param ctx: The context of the command
        :type ctx: discord.Interaction
        :param message: The message that triggered the command
        :type message: discord.Message
        :return: None
        :rtype: None
        """

        # Check if the user is a recruiter and if the user is different from the target user
        if RECRUITER_ROLE_ID not in [role.id for role in ctx.user.roles] and int(
            ctx.user.id
        ) != int(message.author.id):
            return await ctx.respond(BotResponse.NOT_A_RECRUITER.value, ephemeral=True)

        # Check if the target user is in the Applicants group
        if APPLICANT_ROLE_ID not in [role.id for role in message.author.roles]:
            return await ctx.respond(
                BotResponse.MEMBER_NOT_IN_APPLICATION_GROUP.value.format(
                    MEMBER=message.author.mention,
                ),
                ephemeral=True,
            )

        await self.open_ticket(ctx=ctx, member=message.author)

        return None

    @commands.user_command(
        name="Recruit Member", guild_ids=app_settings.get_all_servers()
    )
    async def reverse_recruit_user_context(self, ctx, user):
        """
        Help a new guy get a recruiter

        :param ctx: The context of the command
        :type ctx: discord.Interaction
        :param user: The user that triggered the command
        :type user: discord.User
        :return: None
        :rtype: None
        """

        # Check if the user is a recruiter and if the user is different from the target user
        if RECRUITER_ROLE_ID not in [role.id for role in ctx.user.roles] and int(
            ctx.user.id
        ) != int(user.id):
            return await ctx.respond(BotResponse.NOT_A_RECRUITER.value, ephemeral=True)

        # Check if the target user is in the Applicants group
        if APPLICANT_ROLE_ID not in [role.id for role in user.roles]:
            return await ctx.respond(
                BotResponse.MEMBER_NOT_IN_APPLICATION_GROUP.value.format(
                    MEMBER=user.mention
                ),
                ephemeral=True,
            )

        await self.open_ticket(ctx=ctx, member=user)

        return None


def setup(bot):
    """
    Set up the RecruitMe cog

    :param bot:
    :type bot:
    :return:
    :rtype:
    """

    # Unload the RecruitMe cog from `aadiscordbot`, so we can load our own.
    unload_cog(bot=bot, cog_name="RecruitMe")

    # Add the RecruitMe cog to the bot
    bot.add_cog(RecruitMe(bot))



# Generated by Django 4.2.23 on 2025-07-13 10:14

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tnnt_discordbot_cogs", "0003_setting_locate_channels"),
    ]

    operations = [
        migrations.CreateModel(
            name="Permission",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "Command Permission",
                "permissions": (
                    ("locate", "Can run the `/locate` command"),
                    ("lookup", "Can run the `/lookup` command"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
    ]



# Generated by Django 4.2.23 on 2025-07-10 15:34

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aadiscordbot", "0016_authbotconfiguration_admin_user_ids_and_more"),
        ("tnnt_discordbot_cogs", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="setting",
            name="lookup_channels",
            field=models.ManyToManyField(
                blank=True,
                help_text="Channels in which the `/lookup` command can be used.",
                related_name="lookup_channels",
                to="aadiscordbot.channels",
                verbose_name="Lookup Channels",
            ),
        ),
    ]



"""
"Locator" cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot
"""

# Standard Library
import logging

# Third Party
from corptools.models import EveItemType, MapSystem
from corptools.providers import esi
from discord import Colour, Embed, option
from discord.ext import commands
from pendulum.datetime import DateTime

# Django
from django.core.exceptions import ObjectDoesNotExist

# Alliance Auth
from allianceauth.eveonline.evelinks import dotlan, evewho
from allianceauth.eveonline.models import EveCharacter
from esi.models import Token

# Alliance Auth Discord Bot
from aadiscordbot.app_settings import get_all_servers
from aadiscordbot.cogs.utils.autocompletes import search_characters
from aadiscordbot.cogs.utils.decorators import message_in_channels, sender_has_perm

# Terra Nanotech Discordbot Cogs
from tnnt_discordbot_cogs.helper import unload_cog
from tnnt_discordbot_cogs.models.setting import Setting

logger = logging.getLogger(__name__)


class Locator(commands.Cog):
    """
    A cog to locate characters and their alts in EVE Online.
    """

    def __init__(self, bot):
        """
        Initializes the Locator cog.

        :param bot: The bot instance to which this cog is added.
        :type bot: discord.ext.commands.Bot
        """

        self.bot = bot

    @staticmethod
    def _get_locate_channels() -> list:
        """
        Get the locate channels from the settings.

        :return: List of locate channels or an empty list if none are set.
        :rtype: dict
        """

        locate_channels = Setting.get_setting(Setting.Field.LOCATE_CHANNELS.value).all()

        return (
            [channel.channel for channel in locate_channels if channel is not None]
            if locate_channels
            else []
        )

    @staticmethod
    def _get_locate_embeds(char: EveCharacter) -> list[Embed]:
        """
        Generates embeds for the character's alts' locations.

        :param char: The character for which to generate the embeds.
        :type char: allianceauth.eveonline.models.EveCharacter
        :return: A list of Discord embeds containing the location information of the character's alts.
        :rtype: list[discord.Embed]
        """

        def _process_alt_list(
            header: str, characters: list[dict], color: Colour = Colour.default()
        ) -> list[Embed]:
            """
            Processes a list of alt characters and generates embeds for them.

            :param header: The header for the embed.
            :type header: str
            :param characters: The list of characters to process.
            :type characters: list[dict]
            :param color: The color for the embed.
            :type color: discord.Colour
            :return: A list of Discord embeds containing the alt character information.
            :rtype: list[discord.Embed]
            """

            embeds = []

            # Process alts in chunks of 10
            for alt_grp in [
                characters[i : i + 10] for i in range(0, len(characters), 10)
            ]:
                altstr = []

                for a in alt_grp:
                    evewho_character = evewho.character_url(eve_id=a["cid"])
                    evewho_corporation = evewho.corporation_url(eve_id=a["crpid"])

                    character_line = f"[{a['cnm']}]({evewho_character}) [[{a['crpnm']}]({evewho_corporation})]"

                    if a["lookup"]:
                        dotlan_system = dotlan.solar_system_url(name=a["system_name"])
                        altstr.append(
                            f"### {character_line}\n"
                            f"**Current Location:** [{a['system_name']}]({dotlan_system})\n"
                            f"**Currently Flying:** {a['ship']}\n"
                            f"**Last Online:** {a['last_online'].strftime('%Y-%m-%d %H:%M')}\n"
                        )
                    else:
                        altstr.append(character_line + "\n")

                embed = Embed(title=header, description="\n".join(altstr), colour=color)
                embeds.append(embed)

            return embeds

        alts = char.character_ownership.user.character_ownerships.all().select_related(
            "character"
        )

        alt_online = []
        alt_offline = []
        alt_no_token = []

        for alt in alts:
            _alt = {
                "cid": alt.character.character_id,
                "cnm": alt.character.character_name,
                "crpid": alt.character.corporation_id,
                "crpnm": alt.character.corporation_name,
                "last_online": DateTime.min,
                "online": False,
                "ship": "",
                "system": None,
                "lookup": False,
            }
            scopes = [
                # Locations
                "esi-location.read_location.v1",
                "esi-location.read_online.v1",
                "esi-location.read_ship_type.v1",
            ]

            token = Token.get_token(
                character_id=alt.character.character_id, scopes=scopes
            )

            if token:
                try:
                    # Get all location data in one go
                    character_id = alt.character.character_id
                    valid_token = token.valid_access_token()

                    online = esi.client.Location.get_characters_character_id_online(
                        character_id=character_id,
                        token=valid_token,
                    ).result()

                    location = esi.client.Location.get_characters_character_id_location(
                        character_id=character_id,
                        token=valid_token,
                    ).result()

                    ship = esi.client.Location.get_characters_character_id_ship(
                        character_id=character_id,
                        token=valid_token,
                    ).result()

                    # Update alt data
                    _alt["online"] = (
                        "**Online**" if online.get("online", False) else "**Offline**"
                    )
                    _alt["last_online"] = online.get("last_logout", DateTime.min)

                    solar_system = MapSystem.objects.get(
                        system_id=location["solar_system_id"]
                    )
                    _alt["system_name"] = solar_system.name

                    current_ship, _ = EveItemType.objects.get_or_create_from_esi(
                        ship["ship_type_id"]
                    )
                    _alt["ship"] = current_ship
                    _alt["lookup"] = True

                    if online.get("online", False):
                        alt_online.append(_alt)
                    else:
                        alt_offline.append(_alt)

                except Exception as e:
                    logger.error(
                        f"Error fetching location data for character {alt.character.character_name}: {e}"
                    )
                    alt_no_token.append(_alt)
            else:
                alt_no_token.append(_alt)

        out_embeds = []

        # Process each category if it has items
        for header, alt_list, color in [
            ("Online Characters", alt_online, Colour.green()),
            ("Offline Characters", alt_offline, Colour.orange()),
            ("No Tokens", alt_no_token, Colour.red()),
        ]:
            if alt_list:
                out_embeds += _process_alt_list(
                    header=header, characters=alt_list, color=color
                )

        return out_embeds

    @message_in_channels(channels=_get_locate_channels())
    @sender_has_perm("tnnt_discordbot_cogs.locate")
    @commands.slash_command(name="locate", guild_ids=get_all_servers())
    @option(
        name="character",
        description="Search for a Character!",
        autocomplete=search_characters,
    )
    async def locate(self, ctx, character: str):
        """
        Slash command to locate a character and their alts in EVE Online.

        :param ctx:
        :type ctx:
        :param character:
        :type character:
        :return:
        :rtype:
        """

        try:
            char = EveCharacter.objects.get(character_name=character)
        except EveCharacter.DoesNotExist:
            return await ctx.respond(
                f"Character **{character}** does not exist in our Auth system",
                ephemeral=True,
            )

        try:
            main = char.character_ownership.user.profile.main_character
        except ObjectDoesNotExist:
            return await ctx.respond(
                f"Character **{character}** Unlinked in auth", ephemeral=True
            )

        try:
            discord_string = f"<@{char.character_ownership.user.discord.uid}>"
        except Exception as e:
            logger.error(e)
            discord_string = "unknown"

        await ctx.respond(
            (
                f"Looking up the location of all known alts of {main} ({discord_string})\n"
                "Please Wait..."
            ),
            ephemeral=True,
        )

        embeds = self._get_locate_embeds(char)

        for e in embeds:
            await ctx.respond(embed=e, ephemeral=True)


def setup(bot):
    # Unload the Members cog from `aadiscordbot`
    unload_cog(bot=bot, cog_name="Locator")

    # Add the Lookup cog to the bot
    bot.add_cog(Locator(bot))



"""
Market Price Checks cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot
"""

# Standard Library
import locale
import logging

# Third Party
import requests
from discord import Color, Embed, SlashCommandGroup
from discord.ext import commands
from eve_sde.models import ItemType

# Alliance Auth Discord Bot
from aadiscordbot import app_settings

# Terra Nanotech Discordbot Cogs
from tnnt_discordbot_cogs import __user_agent__
from tnnt_discordbot_cogs.helper import unload_cog

logger = logging.getLogger(__name__)


class PriceCheck(commands.Cog):
    """
    Price checks on Jita, Amarr, Rens, Hek and Dodixie markets
    """

    imageserver_url = "https://images.evetech.net"

    price_commands = SlashCommandGroup(
        "price", "Server Admin Commands", guild_ids=app_settings.get_all_servers()
    )

    def __init__(self, bot):
        """
        Initialize the PriceCheck cog

        :param bot:
        :type bot:
        """

        self.bot = bot

    @staticmethod
    def _get_plex_market():
        """
        Get the PLEX market data

        :return:
        :rtype:
        """

        return [{"name": "Global PLEX Market", "region_id": 19000001}]

    @staticmethod
    def _get_market_data(market: dict, eve_type_id: str) -> dict | None:
        """
        Get the market data for a specific system and item type
        This method fetches market data from the Fuzzwork API.
        The Fuzzwork API provides aggregated market data for EVE Online.
        https://market.fuzzwork.co.uk/aggregates/

        :param market: The market dictionary containing system ID and name
        :type market: dict
        :param eve_type_id: The EVE type ID of the item to fetch data for
        :type eve_type_id: str
        :return: Market data as a json or None if the request fails
        :rtype: Optional[dict]
        """

        market_system_id = market.get("system_id", None)
        market_region_id = market.get("region_id", None)

        url = "https://market.fuzzwork.co.uk/aggregates/"

        url_params = {
            "region" if market_region_id else "system": market_region_id
            or market_system_id,
            "types": eve_type_id,
        }

        request_headers = {"User-Agent": __user_agent__}

        try:
            response = requests.get(
                url=url, params=url_params, headers=request_headers, timeout=5.00
            )
            response.raise_for_status()

            return response.json()
        except requests.RequestException as e:
            logger.error(f"Failed to fetch market data: {e}")

            return None

    @classmethod
    def _build_market_price_embed(
        cls, embed: Embed, market: dict, item_name: str, eve_type_id: str
    ) -> None:
        """
        Build the price embed for the selected market

        :param embed:
        :type embed:
        :param market:
        :type market:
        :param item_name:
        :type item_name:
        :param eve_type_id:
        :type eve_type_id:
        :return:
        :rtype:
        """

        market_name = market["name"]
        market_data = cls._get_market_data(market=market, eve_type_id=eve_type_id)

        embed.add_field(
            name=f"{market_name}",
            value=f"Prices for {item_name} on the {market_name}.",
            inline=False,
        )

        if market_data:
            sell_min = market_data[eve_type_id]["sell"]["min"]
            sell_order_count = int(market_data[eve_type_id]["sell"]["orderCount"])
            buy_max = market_data[eve_type_id]["buy"]["max"]
            buy_order_count = int(market_data[eve_type_id]["buy"]["orderCount"])
            thumbnail_url = f"{cls.imageserver_url}/types/{eve_type_id}/icon?size=64"

            # Set the Embed thumbnail
            embed.set_thumbnail(url=thumbnail_url)

            locale.setlocale(category=locale.LC_ALL, locale="")

            # Sell order price
            market_min_sell_order_price = locale.format_string(
                f="%.2f", val=float(sell_min), grouping=True
            )

            sell_orders_string = f"{sell_order_count} Orders"
            if sell_order_count == 0:
                market_min_sell_order_price = "No sell orders found"
                sell_orders_string = "No Orders"

            if 0 < sell_order_count < 2:
                sell_orders_string = f"{sell_order_count} Order"

            embed.add_field(
                name=f"Sell Order Price ({sell_orders_string})",
                value=f"{market_min_sell_order_price} ISK",
                inline=True,
            )

            # Buy order price
            market_max_buy_order_price = locale.format_string(
                f="%.2f", val=float(buy_max), grouping=True
            )

            buy_orders_string = f"{buy_order_count} Orders"
            if buy_order_count == 0:
                market_max_buy_order_price = "No buy orders found"
                buy_orders_string = "No Orders"

            if 0 < buy_order_count < 2:
                buy_orders_string = f"{buy_order_count} Order"

            embed.add_field(
                name=f"Buy Order Price ({buy_orders_string})",
                value=f"{market_max_buy_order_price} ISK",
                inline=True,
            )
        else:
            embed.add_field(
                name="API Error",
                value=(f"Could not not fetch the price for the {market_name} market."),
                inline=False,
            )

    @classmethod
    def _price_check(cls, markets, item_name: str = None) -> Embed:
        """
        Check the price of an item on the specified markets

        :param markets:
        :type markets:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        if item_name:
            # Special case: PLEX market
            if item_name.lower() == "plex":
                item_name = "PLEX"
                markets = cls._get_plex_market()

            try:
                eve_type_id = str(
                    ItemType.objects.filter(name__iexact=item_name).values_list(
                        "id", flat=True
                    )[0]
                )
            except (ItemType.DoesNotExist, IndexError):
                embed = Embed(
                    title=f"Price Lookup for {item_name}",
                    color=Color.orange(),
                )

                embed.add_field(
                    name="Error",
                    value=(
                        f"{item_name} could not be found. "
                        "Are you sure you spelled it correctly?"
                    ),
                    inline=False,
                )
            else:
                embed = Embed(
                    title=f"Price Lookup: {item_name}",
                    color=Color.green(),
                )

                for market in markets:
                    cls._build_market_price_embed(
                        embed=embed,
                        market=market,
                        item_name=item_name,
                        eve_type_id=eve_type_id,
                    )
        else:
            embed = Embed(
                title="Price Lookup",
                color=Color.red(),
            )

            embed.add_field(
                name="Error",
                value=(
                    "You forget to enter an item you want to lookup the price for ..."
                ),
                inline=False,
            )

        return embed

    @price_commands.command(
        name="all_markets",
        description="Check an item price on all major market hubs",
    )
    async def price(self, ctx, item_name: str):
        """
        Check an item price on all major market hubs

        :param ctx:
        :type ctx:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        return await ctx.respond(
            embed=self._price_check(
                markets=[
                    {"name": "Jita Market", "system_id": 30000142},
                    {"name": "Amarr Market", "system_id": 30002187},
                    {"name": "Rens Market", "system_id": 60004588},
                    {"name": "Hek Market", "system_id": 60005686},
                    {"name": "Dodixie Market", "system_id": 30002659},
                ],
                item_name=item_name,
            ),
            ephemeral=True,
        )

    @price_commands.command(
        name="jita",
        description="Check an item price on Jita market",
    )
    async def jita(self, ctx, item_name: str):
        """
        Check an item price on Jita market

        :param ctx:
        :type ctx:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        return await ctx.respond(
            embed=self._price_check(
                markets=[{"name": "Jita Market", "system_id": 30000142}],
                item_name=item_name,
            ),
            ephemeral=True,
        )

    @price_commands.command(
        name="amarr",
        description="Check an item price on Amarr market",
    )
    async def amarr(self, ctx, item_name: str):
        """
        Check an item price on Amarr market

        :param ctx:
        :type ctx:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        return await ctx.respond(
            embed=self._price_check(
                markets=[{"name": "Amarr Market", "system_id": 60008494}],
                item_name=item_name,
            ),
            ephemeral=True,
        )

    @price_commands.command(
        name="rens",
        description="Check an item price on Rens market",
    )
    async def rens(self, ctx, item_name: str):
        """
        Check an item price on Rens market

        :param ctx:
        :type ctx:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        return await ctx.respond(
            embed=self._price_check(
                markets=[{"name": "Rens Market", "system_id": 60004588}],
                item_name=item_name,
            ),
            ephemeral=True,
        )

    @price_commands.command(
        name="hek",
        description="Check an item price on Hek market",
    )
    async def hek(self, ctx, item_name: str):
        """
        Check an item price on Hek market

        :param ctx:
        :type ctx:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        return await ctx.respond(
            embed=self._price_check(
                markets=[{"name": "Hek Market", "system_id": 60005686}],
                item_name=item_name,
            ),
            ephemeral=True,
        )

    @price_commands.command(
        name="dodixie",
        description="Check an item price on Dodixie market",
    )
    async def dodixie(self, ctx, item_name: str):
        """
        Check an item price on Dodixie market

        :param ctx:
        :type ctx:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        return await ctx.respond(
            embed=self._price_check(
                markets=[{"name": "Dodixie Market", "system_id": 30002659}],
                item_name=item_name,
            ),
            ephemeral=True,
        )

    @price_commands.command(
        name="plex",
        description="Check the PLEX price on the global PLEX market",
    )
    async def plex(self, ctx):
        """
        Check the PLEX price on the global PLEX market

        :param ctx:
        :type ctx:
        :param item_name:
        :type item_name:
        :return:
        :rtype:
        """

        return await ctx.respond(
            embed=self._price_check(
                markets=[],  # PLEX market is a special case, so keep it empty, it will be set later in the method
                item_name="PLEX",
            ),
            ephemeral=True,
        )


def setup(bot) -> None:
    """
    Set up the cog

    :param bot:
    :type bot:
    :return:
    :rtype:
    """

    # Unload the PriceCheck cog from `aadiscordbot`, so we can load our own.
    unload_cog(bot=bot, cog_name="PriceCheck")

    # Add the PriceCheck cog to the bot
    bot.add_cog(PriceCheck(bot))



"""
"About" cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot

Since we don't want to have it branded for "The Initiative" we have to build our own
"""

# Standard Library
import logging

# Third Party
from discord import Color, Embed
from discord.ext import commands

# Django
from django.conf import settings

# Alliance Auth
from allianceauth.eveonline.evelinks.eveimageserver import (
    alliance_logo_url,
    corporation_logo_url,
)

# Alliance Auth Discord Bot
from aadiscordbot.app_settings import get_site_url

# Terra Nanotech Discordbot Cogs
from tnnt_discordbot_cogs.helper import unload_cog

logger = logging.getLogger(name=__name__)


class About(commands.Cog):
    """
    All about me!
    """

    def __init__(self, bot):
        """
        Initialize the About cog.

        :param bot:
        :type bot:
        """

        self.bot = bot

    @commands.slash_command(
        name="about",
        description="All about the bot",
        guild_ids=[int(settings.DISCORD_GUILD_ID)],
    )
    async def about(self, ctx):
        """
        All about the bot

        :param ctx:
        :type ctx:
        :return:
        :rtype:
        """

        await ctx.defer()

        auth_url = get_site_url()
        embed = Embed(title="TN-NT Discord Services")

        try:
            if settings.TNNT_TEMPLATE_ENTITY_ID == 1:
                aa_icon = f"{auth_url}/static/allianceauth/icons/allianceauth.png"
                embed.set_thumbnail(url=aa_icon)
            else:
                if settings.TNNT_TEMPLATE_ENTITY_TYPE == "alliance":
                    embed.set_thumbnail(
                        url=alliance_logo_url(
                            alliance_id=settings.TNNT_TEMPLATE_ENTITY_ID, size=256
                        )
                    )
                elif settings.TNNT_TEMPLATE_ENTITY_TYPE == "corporation":
                    embed.set_thumbnail(
                        url=corporation_logo_url(
                            corporation_id=settings.TNNT_TEMPLATE_ENTITY_ID, size=256
                        )
                    )
        except AttributeError:
            pass

        embed.colour = Color.green()
        embed.description = "This is a multi-functional discord bot tailored specifically for Terra Nanotech."
        embed.add_field(name="Auth Link", value=auth_url, inline=False)
        embed.set_footer(
            text="Developed by Aaron Kable and Ariel Rin, forked for Terra Nanotech by Rounon Dax"
        )

        return await ctx.respond(embed=embed)


def setup(bot):
    """
    Set up the cog

    :param bot:
    :type bot:
    :return:
    :rtype:
    """

    # Unload the About cog from `aadiscordbot`, so we can load our own.
    unload_cog(bot=bot, cog_name="About")

    # Add the About cog to the bot
    bot.add_cog(About(bot))



"""
Hooking into the auth system
"""

# Alliance Auth
from allianceauth import hooks


@hooks.register("discord_cogs_hook")
def register_cogs():
    """
    Registering our discord cogs
    :return:
    :rtype:
    """

    return [
        "tnnt_discordbot_cogs.cogs.about",
        "tnnt_discordbot_cogs.cogs.admin",
        "tnnt_discordbot_cogs.cogs.auth",
        "tnnt_discordbot_cogs.cogs.locate",
        "tnnt_discordbot_cogs.cogs.lookup",
        "tnnt_discordbot_cogs.cogs.models",
        "tnnt_discordbot_cogs.cogs.price_check",
        "tnnt_discordbot_cogs.cogs.recruit_me",
        "tnnt_discordbot_cogs.cogs.welcome",
    ]



"""
"Welcome" cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot
"""

# Standard Library
import asyncio
import logging

# Third Party
from discord import Member
from discord.ext import commands

# Django
from django.db.models import Q

# Alliance Auth Discord Bot
from aadiscordbot.app_settings import get_site_url
from aadiscordbot.models import WelcomeMessage
from aadiscordbot.utils.auth import is_user_authenticated

# Terra Nanotech Discordbot Cogs
from tnnt_discordbot_cogs.helper import unload_cog
from tnnt_discordbot_cogs.models.setting import Setting

logger = logging.getLogger(__name__)

# Discord Bot Settings
# Channel
WELCOME_CHANNEL_AUTHENTICATED = Setting.get_setting(
    Setting.Field.WELCOME_CHANNEL_AUTHENTICATED.value
).channel
WELCOME_CHANNEL_UNAUTHENTICATED = Setting.get_setting(
    Setting.Field.WELCOME_CHANNEL_UNAUTHENTICATED.value
).channel

# Excluded Roles
WELCOME_ROLES_EXCLUDED = Setting.get_setting(
    Setting.Field.WELCOME_ROLES_EXCLUDED.value
).split(",")


class Welcome(commands.Cog):
    """
    Responds to on_member_join events from discord
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.Cog.listener("on_member_join")
    async def on_member_join(self, member: Member):
        """
        Responds to on_member_join events from discord

        :param member: discord.Member object representing the new member
        :type member: discord.Member
        :return: None
        :rtype: None
        """

        logger.info(msg=f"{member} joined {member.guild.name}")

        # Default to the system channel if no channel is configured.
        channel = member.guild.system_channel

        try:
            # Give AA a chance to save the UID for a joiner.
            await asyncio.sleep(3)

            authenticated = is_user_authenticated(user=member, guild=member.guild)
        except Exception:
            authenticated = False

        # If the user is authenticated, send a welcome message to the authenticated channel.
        if authenticated:
            if any(role.name in WELCOME_ROLES_EXCLUDED for role in member.roles):
                # If the channel_id is an integer, get the channel object.
                if isinstance(WELCOME_CHANNEL_AUTHENTICATED, int):
                    channel = member.guild.get_channel(WELCOME_CHANNEL_AUTHENTICATED)

                try:
                    message = (
                        WelcomeMessage.objects.filter(
                            Q(Q(guild_id=member.guild.id) | Q(guild_id=None)),
                            authenticated=True,
                        )
                        .order_by("?")
                        .first()
                        .message
                    )
                    message_formatted = message.format(
                        user_mention=member.mention,
                        guild_name=member.guild.name,
                        auth_url=get_site_url(),
                    )

                    await channel.send(content=message_formatted)
                except IndexError:
                    logger.error(
                        msg="No Welcome Message configured for Discordbot Welcome cog"
                    )
                except Exception as e:
                    logger.error(msg=e)
        # If the user is not authenticated, send a welcome message to the unauthenticated channel.
        else:
            # If the channel_id is an integer, get the channel object.
            if isinstance(WELCOME_CHANNEL_UNAUTHENTICATED, int):
                channel = member.guild.get_channel(WELCOME_CHANNEL_UNAUTHENTICATED)

            try:
                message = (
                    WelcomeMessage.objects.filter(
                        Q(Q(guild_id=member.guild.id) | Q(guild_id=None)),
                        unauthenticated=True,
                    )
                    .order_by("?")
                    .first()
                    .message
                )
                message_formatted = message.format(
                    user_mention=member.mention,
                    guild_name=member.guild.name,
                    auth_url=get_site_url(),
                )

                await channel.send(message_formatted)
            except IndexError:
                logger.error(
                    msg="No Welcome Message configured for Discordbot Welcome cog"
                )
            except Exception as e:
                logger.error(msg=e)


def setup(bot):
    """
    Set up the Welcome cog

    :param bot: discord bot
    """

    # Unload the Welcome cog from `aadiscordbot`, so we can load our own.
    unload_cog(bot=bot, cog_name="Welcome")

    # Add the Welcome cog to the bot
    bot.add_cog(Welcome(bot))



"""
Helper functions for the TNNT Discord bot.
"""

# Standard Library
from urllib.parse import urljoin

# Third Party
from discord.ext import commands

# Django
from django.conf import settings
from django.urls import reverse


def unload_cog(bot: commands.Bot, cog_name: str) -> None:
    """
    Unloads a cog from the bot.

    :param bot: The bot instance.
    :type bot: discord.ext.commands.Bot
    :param cog_name: The name of the cog to unload.
    :type cog_name: str
    """

    try:
        bot.remove_cog(name=cog_name)

        print(f"Unloaded {cog_name} successfully.")
    except Exception as e:
        print(f"Failed to unload {cog_name}: {e}")


def reverse_absolute(viewname: str, args: list | None = None) -> str:
    """
    Return absolute URL for a view name.

    Similar to Django's ``reverse()``, but returns an absolute URL.
    """

    return urljoin(base=settings.SITE_URL, url=reverse(viewname=viewname, args=args))
